!---------------------------------------------------------------------------------------
! FLEXINVERT: mod_states
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_states
!!
!! Purpose:    Module for constructing the states data structure.
!!
!---------------------------------------------------------------------------------------


module mod_states

  use mod_settings
  use mod_var

  implicit none
  private

  public :: states_t, alloc_states, dealloc_states

  type :: states_t
 
    real, dimension(:), allocatable         :: px       ! state vector 
    real, dimension(:), allocatable         :: px0      ! prior state vector 
    real, dimension(:), allocatable         :: pxerr0   ! prior uncertainty
    real, dimension(:), allocatable         :: pxerr    ! posterior uncertainty 
    real(kind=8), dimension(:), allocatable :: xtime    ! time stamp of state vector (julian days)
    real(kind=8), dimension(:), allocatable :: cinitime ! time stamp of initial mixing ratio scalars (julian days)

  end type

  contains

  ! --------------------------------------------------
  ! alloc_states
  ! --------------------------------------------------

  subroutine alloc_states(states)

    type (states_t),      intent(in out) :: states
    integer                              :: ierr

    allocate( states%px(nvar), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
    allocate( states%px0(nvar), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
    allocate( states%pxerr0(nvar), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
    allocate( states%pxerr(nvar), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
    allocate( states%xtime(ntstate), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
    allocate( states%cinitime(ntcini), stat = ierr )
    if ( ierr.ne.0 ) then
      write(logid,*) 'ERROR alloc_states: not enough memory'
      stop
    endif
 
  end subroutine alloc_states

  ! --------------------------------------------------
  ! dealloc_states
  ! --------------------------------------------------

  subroutine dealloc_states(states)

    type (states_t),      intent(in out) :: states

    if ( allocated(states%px) )       deallocate(states%px)
    if ( allocated(states%px0) )      deallocate(states%px0)
    if ( allocated(states%pxerr0) )   deallocate(states%pxerr0)
    if ( allocated(states%pxerr) )    deallocate(states%pxerr)
    if ( allocated(states%xtime) )    deallocate(states%xtime)
    if ( allocated(states%cinitime) ) deallocate(states%cinitime)

  end subroutine dealloc_states

  ! --------------------------------------------------

end module mod_states




